unit SWavLoad;

interface

const
     HdrSize=44;

type
    TWavLoader = class
        FileName:string;
        Buffer,Start:PChar;
        WholeLen,Len,Log2Len:Integer;
        ErrorNotFound,ErrorFormat,ErrorData:Boolean;
        function isError:Boolean;
        constructor Create (fn:string);
        procedure ReadWAVEfmt;
        procedure Readdata;
        function FindStr (s:string):PChar;
        destructor Destroy; override;
        constructor Clone (t:TObject);
    end;
    BigData = array [0..100000] of Char;
    PBigData = ^BigData;

var
       WavHdr:array [0..HdrSize-1] of Byte
    =(
    $52,$49,$46,$46,  0,  0,  0,  0,$57,$41,$56,$45,$66,$6d,$74,$20,
    $10,$00,$00,$00,$01,$00, $01,$00,$22,$56,$00,$00,$22,$56,$00,$00,
    $01,$00,$08,$00, $64,$61,$74,$61,  0,  0,  0,  0
    );


implementation

uses Dialogs,SysUTils;

function TWavLoader.isError:Boolean;
begin
     result:=ErrorNotFound or ErrorFormat or ErrorData;
end;

function TWavLoader.FindStr (s:string):PChar;
var i,l:Integer;
begin
     l:=Length(s);
     result:=Buffer;
     while result<Buffer+WholeLen-l do begin
           i:=0;
           while i<l do begin
               if (result+i)^<>s[i+1] then break;
               inc (i);
           end;
           if i>=l then exit;
           inc (result);
     end;
     result:=nil;

end;

procedure TWavLoader.ReadWAVEfmt;
var s:string;
    p:PChar;
    i,l:Integer;
begin
     ErrorFormat:=True;
     p:=FindStr ('WAVEfmt ');
     if p<>nil then begin
         inc (p,8); // Skip "Wavefmt "
         inc (p,6); // Skip HdrLength,Tag
         for i:=0 to 13 do begin
             if Byte((p+i)^)<>WavHdr[i+22] then break;
             if i=13 then ErrorFormat:=False;
         end;
     end;
end;

procedure TWavLoader.Readdata;
var s:string;
    p:PChar;
    i,l:Integer;
begin
     ErrorData:=True;
     p:=FindStr('data');
     if p<>nil then begin
         inc (p,4);   // Skip "data"
         Len:=0;
         {Len:=Len or ( Byte((p+0)^) shl 0 );
         Len:=Len or ( Byte((p+1)^) shl 8 );
         Len:=Len or ( Byte((p+2)^) shl 16 );
         Len:=Len or ( Byte((p+3)^) shl 24 );}
         Start:=p+4;
         //ShowMessage (IntTostr  (Integer(Pointer(Start)))+'**' );

         Len:=Buffer+WholeLen-Start;
         ErrorData:=False;
         //if Start+Len=Buffer+WholeLen then ErrorData:=False;

     end;
end;

destructor TWavLoader.Destroy;
begin
     inherited Destroy;
     FreeMem (Buffer);
end;

constructor TWavLoader.Clone (t:TObject);
var tt:TWavLoader;
    i,c:Integer;
begin
     tt:=TWavLoader(t);
     Len:=1;Log2Len:=0;
     while Len<tt.Len do begin
           inc(Log2Len);
           Len:=Len*2;
     end;
     GetMem (Buffer,Len);
     Start:=Buffer;
     i:=0;c:=Len;
     {ShowMessage (IntTostr  (Integer(Pointer(Start)))+'//'
                  +IntTostr (Integer(Pointer(tt.Start))) );}
     //try
     while i<c do begin
         if i<tt.Len-1 then (Start+i)^:=(tt.Start+i)^
         else (Start+i)^:=Chr(128);//(tt.Start+tt.Len-1)^;
         inc(i);
     end;
     {except
          ShowMessage (IntTostr (i)+'//');

     end;    }
end;

constructor TWavLoader.Create (fn:string);
var f:file;
begin
     if fn='' then exit;

     ErrorNotFound:=False;
     try
       AssignFile (f,fn);
       ReSet (f,1);
     except
       ErrorNotFound:=True;
     end;
     if ErrorNotFound then exit;
     WholeLen:=FileSize (f);
     GetMem (Buffer,WholeLen);
     BlockRead (f,PBigData(Buffer)^,WholeLen);
     CloseFile (f);

     ReadWAVEfmt;
     ReadData;

     {if not ErrorFormat and not ErrorData then begin
         ShowMessage ('Success '+fn+' : Length='+IntToStr(Len));
     end; }
end;


end.
